/******************************************************************************************************/
/* The 1st IPSJ SIG-ARC High-Performance Processor Design Contest                     Version 2013-10 */
/* From MieruPC 2010: Top Level Module                                            ArchLab. TOKYO TECH */
/******************************************************************************************************/
`include "../rtl/define.v"
`default_nettype none

/******************************************************************************************************/
module MieruSys(CLK_IN, RST_X_IN, RXD, TXD, ULED,
    DDR2CLK0, DDR2CLK1, DDR2CKE, DDR2RASN, DDR2CASN, DDR2WEN, DDR2RZQ, DDR2BA, DDR2A, DDR2DQ, 
    DDR2UDQS, DDR2UDQSN, DDR2LDQS, DDR2LDQSN, DDR2LDM, DDR2UDM, DDR2ODT, DDR2ZIO);
      
    output DDR2CLK0, DDR2CLK1, DDR2CKE, DDR2RASN, DDR2CASN, DDR2WEN, DDR2RZQ, DDR2ZIO;
    output DDR2LDM, DDR2UDM, DDR2ODT;
    output [2:0]  DDR2BA;
    output [12:0] DDR2A;
    inout  [15:0] DDR2DQ;
    inout  DDR2UDQS, DDR2UDQSN, DDR2LDQS, DDR2LDQSN;
            
    input            CLK_IN, RST_X_IN, RXD;
    output reg       TXD;
    output reg [7:0] ULED;

    wire           CLK, DRAM_CLK, RST_X;
    wire [2:0]     CORE_STAT;
    
    wire [31:0]    MM_ADDR;
    wire [7:0]     MM_DATA;
    wire           MM_WE;
    wire [7:0]     MM_DOUT; 
    
    wire [31:0]    INIT_ADDR;
    wire [31:0]    INIT_DATA;
    wire           INIT_WE;
    wire           INIT_DONE;
    
    wire [31:0]    CORE_I_ADDR, CORE_D_ADDR;
    wire [31:0]    CORE_I_IN, CORE_D_IN, CORE_D_OUT;
    wire [3:0]     CORE_D_WE;
    wire           CORE_D_OE; 

    wire clk_t, locked, locked2;
    IBUFG  ibuf (.I(CLK_IN), .O(clk_t));
    clockgen  clockgen01(clk_t, CLK,      locked);
    clockgen2 clockgen02(clk_t, DRAM_CLK, locked2);  
    resetgen resetgen01(CLK, (RST_X_IN & locked & locked2), RST_X);

    /**************************************************************************************************/
    wire [31:0] mem_adr;        //
    wire        mem_we, mem_re; //
    wire [31:0] mem_wr_dat;     //
    wire [31:0] mem_rd_dat;     //
    wire        mem_busy;       //
    wire        calib_done;     //
    
    dram_con mem0(.CLK(CLK), .DRAM_CLK(DRAM_CLK), .RST_X(RST_X), .calib_done(calib_done), 
                  .D_ADR(mem_adr), .D_DIN(mem_wr_dat), .D_WE(mem_we), 
                  .D_RE(mem_re), .D_DOUT(mem_rd_dat), .D_BUSY(mem_busy),
    // DRAM interface
   .DDR2CLK0        (DDR2CLK0),
   .DDR2CLK1        (DDR2CLK1),
   .DDR2CKE         (DDR2CKE),
   .DDR2RASN        (DDR2RASN),
   .DDR2CASN        (DDR2CASN),
   .DDR2WEN         (DDR2WEN),
   .DDR2RZQ         (DDR2RZQ),
   .DDR2ZIO         (DDR2ZIO),
   .DDR2BA          (DDR2BA),
   .DDR2A           (DDR2A),
   .DDR2DQ          (DDR2DQ),
   .DDR2UDQS        (DDR2UDQS),
   .DDR2UDQSN       (DDR2UDQSN),
   .DDR2LDQS        (DDR2LDQS),
   .DDR2LDQSN       (DDR2LDQSN),
   .DDR2LDM         (DDR2LDM),
   .DDR2UDM         (DDR2UDM),
   .DDR2ODT         (DDR2ODT)
    );
    
   /**************************************************************************************************/
    PLOADER loader(.CLK(CLK), .RST_X(RST_X), .RXD(RXD), 
                   .ADDR(INIT_ADDR), .DATA(INIT_DATA), .WE(INIT_WE), .DONE(INIT_DONE));
    
    /**************************************************************************************************/
    wire core_rst_x = RST_X & INIT_DONE;
    wire [31:0] WORD_A, WORD_D;
    wire        WORD_W;
    
    MIPSCORE core(.CLK(CLK), .RST_X(core_rst_x), . STALL(mem_busy), .STAT(CORE_STAT), .IO_IN(0), 
                  .I_ADDR(CORE_I_ADDR), .I_IN(CORE_I_IN), 
                  .D_ADDR(CORE_D_ADDR), .D_IN(CORE_D_IN), .D_OE(CORE_D_OE), 
                  .D_OUT(CORE_D_OUT), .D_WE(CORE_D_WE));

    IMEM imem(CLK, RST_X, INIT_ADDR, INIT_DATA, INIT_WE, CORE_I_ADDR, CORE_I_IN);

    assign mem_adr    = (!INIT_DONE) ? INIT_ADDR : CORE_D_ADDR;
    assign mem_wr_dat = (!INIT_DONE) ? INIT_DATA : CORE_D_OUT;
    assign mem_we     = (!INIT_DONE) ? INIT_WE   : (CORE_D_WE!=0 && CORE_D_ADDR>32'hff); // Note!
    assign mem_re     = CORE_D_OE;
    assign CORE_D_IN  = mem_rd_dat;

    /**************************************************************************************************/
    reg [25:0] cnt_t; // just for ULED
    always @(posedge CLK) cnt_t <= cnt_t + 1;
    
    always @(posedge CLK) ULED[7] <= cnt_t[25];
    always @(posedge CLK) ULED[6] <= CORE_STAT[1];       // Processor Memory Alignment Error
    always @(posedge CLK) ULED[5] <= CORE_STAT[0];       // Processor Decode Error 
    always @(posedge CLK) ULED[4] <= mem_busy;           // DRAM is working
    always @(posedge CLK) ULED[3] <= ~TXD;               // Uart TXD
    always @(posedge CLK) ULED[2] <= ~RXD;               // Uart RXD
    always @(posedge CLK) ULED[1] <= ~calib_done;        // DRAM calibration done 
    always @(posedge CLK) ULED[0] <= ~INIT_DONE;         // MEMORY IMAGE transfer is done

    /**************************************************************************************************/
    wire tx_ready, txd_w;
    always @(posedge CLK) TXD <= txd_w;
    UartTx send(CLK, RST_X, CORE_D_OUT[7:0], (CORE_D_WE!=0 & CORE_D_ADDR==0), txd_w, tx_ready); 
endmodule
/******************************************************************************************************/
